@@ -9,7 +9,7 @@
 #
 
 
-__all__ = ["keygens", "AsyncKeys", "Keys"]
+__all__ = ["keygens", "AsyncKeys", "Keys", "amnemonic", "mnemonic"]
 
 
 __doc__ = """
@@ -17,18 +17,24 @@ A collection of highlevel tools for creating & managing symmetric keys.
 """
 
 
-from .asynchs  import *
-from .commons import primes
-from .commons import commons
+from .asynchs import *
+from .commons import *
+from .randoms import salt
+from .randoms import asalt
 from .randoms import csprng
 from .randoms import acsprng
 from .ciphers import keys
 from .ciphers import akeys
 from .ciphers import subkeys
 from .ciphers import asubkeys
+from .ciphers import passcrypt
+from .ciphers import apasscrypt
+from .ciphers import bytes_keys
+from .ciphers import abytes_keys
 from .ciphers import OneTimePad
 from .ciphers import keypair_ratchets
 from .ciphers import akeypair_ratchets
+from .generics import azip
 from .generics import is_iterable
 from .generics import comprehension
 from .generics import sha_256_hmac
@@ -37,7 +43,7 @@ from .generics import convert_static_method_to_member
 
 
 @comprehension()
-async def atable_key_gen(key=None, table=commons.ASCII_TABLE):
+async def atable_key_gen(key=None, table=ASCII_TABLE):
     """
     This table based key generator function converts any key string
     containing an arbitrary set of characters, into another key string
@@ -84,7 +90,7 @@ async def atable_key_gen(key=None, table=commons.ASCII_TABLE):
 
 
 @comprehension()
-def table_key_gen(key=None, table=commons.ASCII_TABLE):
+def table_key_gen(key=None, table=ASCII_TABLE):
     """
     This table based key generator function converts any key string
     containing an arbitrary set of characters, into another key string
@@ -131,7 +137,7 @@ def table_key_gen(key=None, table=commons.ASCII_TABLE):
         yield table[index % prime % table_size]
 
 
-async def atable_key(key=None, table=commons.ASCII_TABLE, size=64):
+async def atable_key(key=None, table=ASCII_TABLE, size=64):
     """
     This table based key function converts any key string containing
     any arbitrary set of characters, into another key string containing
@@ -167,10 +173,13 @@ async def atable_key(key=None, table=commons.ASCII_TABLE, size=64):
         if type(table) == dict:
             return await generator.alist()
         else:
-            return await generator.ajoin()
+            try:
+                return await generator.ajoin()
+            except TypeError:
+                return await generator.ajoin(b"")
 
 
-def table_key(key=None, table=commons.ASCII_TABLE, size=64):
+def table_key(key=None, table=ASCII_TABLE, size=64):
     """
     This table based key function converts any key string containing
     any arbitrary set of characters, into another key string containing
@@ -209,7 +218,52 @@ def table_key(key=None, table=commons.ASCII_TABLE, size=64):
         if type(table) == dict:
             return generator.list()
         else:
-            return generator.join()
+            try:
+                return generator.join()
+            except TypeError:
+                return generator.join(b"")
+
+
+@comprehension()
+async def amnemonic(password, salt=None, words=WORD_LIST):
+    """
+    Creates a stream of words for a mnemonic key from a user password &
+    random salt. If a salt isn't passed, then a random salt is generated
+    & is available by calling ``result(exit=True)`` on the generator
+    object. The ``words`` used for the mnemonic can be passed in, but by
+    default are a 2048 word list of unique, all lowercase english words.
+    """
+    translate = None
+    length = len(words)
+    salt = salt if salt else await asalt()
+    key = await apasscrypt(password, salt)
+    entropy = abytes_keys(password, salt, key)
+    async with entropy.abytes_to_int().arelay(salt) as indexes:
+        while True:
+            if translate:
+                await entropy.gen.asend(translate)
+            translate = yield words[await indexes() % length]
+
+
+@comprehension()
+def mnemonic(password, salt=None, words=WORD_LIST):
+    """
+    Creates a stream of words for a mnemonic key from a user password &
+    random salt. If a salt isn't passed, then a random salt is generated
+    & is available by calling ``result(exit=True)`` on the generator
+    object. The ``words`` used for the mnemonic can be passed in, but by
+    default are a 2048 word list of unique, all lowercase english words.
+    """
+    translate = None
+    length = len(words)
+    salt = salt if salt else globals()["salt"]()
+    key = passcrypt(password, salt)
+    entropy = bytes_keys(password, salt, key)
+    with entropy.bytes_to_int().relay(salt) as indexes:
+        while True:
+            if translate:
+                entropy.gen.send(translate)
+            translate = yield words[indexes() % length]
 
 
 async def akeypair(entropy=csprng()):
@@ -236,10 +290,13 @@ class AsyncKeys:
     & ``asubkeys`` methods. The class also contains static method key
     generators which function independantly from instance states.
     """
-    aseed = acsprng
+
+    aseed = staticmethod(asalt)
     akeys = staticmethod(akeys)
     asubkeys = staticmethod(asubkeys)
     akeypair = staticmethod(akeypair)
+    apasscrypt = staticmethod(apasscrypt)
+    abytes_keys = staticmethod(abytes_keys)
     atable_key = staticmethod(atable_key)
     atable_key_gen = staticmethod(atable_key_gen)
     akeypair_ratchets = staticmethod(akeypair_ratchets)
@@ -249,8 +306,8 @@ class AsyncKeys:
         Creates a symmetric key pair used to create deterministic streams
         of key material.
         """
-        self.key, self.salt = key, csprng(key) if key else keypair()
-        for method in [akeys, asubkeys]:
+        self.key, self.salt = (key, salt()) if key else keypair()
+        for method in [akeys, asubkeys, abytes_keys]:
             convert_static_method_to_member(
                 self, method.__name__, method, key=self.key, salt=self.salt
             )
@@ -263,7 +320,7 @@ class AsyncKeys:
         return self.akeys(key=self.key, salt=self.salt, pid=salt)
 
     async def ahmac(
-        self, data=None, key=None, *, hasher=asha_256_hmac
+        self, data=None, *, key=None, hasher=asha_256_hmac
     ):
         """
         Creates an HMAC code of ``data`` using ``key`` & the hashing
@@ -272,14 +329,28 @@ class AsyncKeys:
         return await hasher(data, key=key if key else self.key)
 
     async def atest_hmac(
-        self, data=None, hmac=None, key=None, *, hasher=sha_256_hmac
+        self, data=None, hmac=None, *, key=None, hasher=asha_256_hmac
     ):
         """
-        Tests the ``hmac`` code against the derived HMAC of ``data``
-        using ``key`` & the async hashing function ``hasher``.
+        Tests if ``hmac`` of ``data`` is valid using ``key`` or the
+        instance's ``self.key`` if it's not supplied. Instead of using a
+        constant time character by character check on the hmac, the hmac
+        itself is hmac'd & is checked against the hmac of the correct
+        hmac. This non-constant time check on the hmac of the supplied
+        hmac doesn't reveal meaningful information about the true hmac
+        if the attacker does not have access to the secret key. This
+        scheme is easier to implement correctly & is easier to guarantee
+        the infeasibility of a timing attack. An async ``hasher``
+        function can also be supplied to use that algorithm instead of
+        the default ``asha_256_hmac``.
         """
-        if hmac == await self.ahmac(
-            data=data, key=key if key else self.key, hasher=hasher
+        if not hmac:
+            raise ValueError("`hmac` keyword argument was not given.")
+        key = key if key else self.key
+        true_hmac = await self.ahmac(data=data, key=key, hasher=hasher)
+        if (
+            await self.ahmac(hmac, key=key)
+            == await self.ahmac(true_hmac, key=key)
         ):
             return True
         else:
@@ -299,8 +370,8 @@ class AsyncKeys:
         if both:
             self.key, self.salt = await akeypair()
         else:
-            self.salt = await acsprng(csprng())
-        for method in [akeys, asubkeys]:
+            self.salt = await self.aseed()
+        for method in [akeys, asubkeys, abytes_keys]:
             convert_static_method_to_member(
                 self, method.__name__, method, key=self.key, salt=self.salt
             )
@@ -314,10 +385,13 @@ class Keys:
     & ``subkeys`` methods. The class also contains static method key
     generators which function independantly from instance states.
     """
-    seed = csprng
+
+    seed = staticmethod(salt)
     keys = staticmethod(keys)
     subkeys = staticmethod(subkeys)
     keypair = staticmethod(keypair)
+    passcrypt = staticmethod(passcrypt)
+    bytes_keys = staticmethod(bytes_keys)
     table_key = staticmethod(table_key)
     table_key_gen = staticmethod(table_key_gen)
     keypair_ratchets = staticmethod(keypair_ratchets)
@@ -337,7 +411,7 @@ class Keys:
         """
         return self.keys(key=self.key, salt=self.salt, pid=salt)
 
-    def hmac(self, data=None, key=None, *, hasher=sha_256_hmac):
+    def hmac(self, data=None, *, key=None, hasher=sha_256_hmac):
         """
         Creates an HMAC code of ``data`` using ``key`` & the hashing
         function ``hasher``.
@@ -345,15 +419,26 @@ class Keys:
         return hasher(data, key=key if key else self.key)
 
     def test_hmac(
-        self, data=None, hmac=None, key=None, *, hasher=sha_256_hmac
+        self, data=None, hmac=None, *, key=None, hasher=sha_256_hmac
     ):
         """
-        Tests the ``hmac`` code against the derived HMAC of ``data``
-        using ``key`` & the hashing function ``hasher``.
+        Tests if ``hmac`` of ``data`` is valid using ``key`` or the
+        instance's ``self.key`` if it's not supplied. Instead of using a
+        constant time character by character check on the hmac, the hmac
+        itself is hmac'd & is checked against the hmac of the correct
+        hmac. This non-constant time check on the hmac of the supplied
+        hmac doesn't reveal meaningful information about the true hmac
+        if the attacker does not have access to the secret key. This
+        scheme is easier to implement correctly & is easier to guarantee
+        the infeasibility of a timing attack. A sync ``hasher`` function
+        can also be supplied to use that algorithm instead of the
+        default ``sha_256_hmac``.
         """
-        if hmac == self.hmac(
-            data=data, key=key if key else self.key, hasher=hasher
-        ):
+        if not hmac:
+            raise ValueError("`hmac` keyword argument was not given.")
+        key = key if key else self.key
+        true_hmac = self.hmac(data=data, key=key, hasher=hasher)
+        if self.hmac(hmac, key=key) == self.hmac(true_hmac, key=key):
             return True
         else:
             raise ValueError("HMAC of ``data`` isn't valid.")
@@ -372,8 +457,8 @@ class Keys:
         if both:
             self.key, self.salt = keypair()
         else:
-            self.salt = csprng(csprng())
-        for method in [keys, subkeys]:
+            self.salt = self.seed()
+        for method in [keys, subkeys, bytes_keys]:
             convert_static_method_to_member(
                 self, method.__name__, method, key=self.key, salt=self.salt
             )
@@ -409,12 +494,14 @@ __extras = {
     "insert_keyrings": insert_keyrings,
     "akeys": akeys,
     "keys": keys,
+    "mnemonic": mnemonic,
     "asubkeys": asubkeys,
     "subkeys": subkeys,
     "akeypair": akeypair,
     "keypair": keypair,
     "akeypair_ratchets": akeypair_ratchets,
     "keypair_ratchets": keypair_ratchets,
+    "amnemonic": amnemonic,
     "atable_key": atable_key,
     "table_key": table_key,
     "atable_key_gen": atable_key_gen,
@@ -422,5 +509,5 @@ __extras = {
 }
 
 
-keygens = commons.Namespace.make_module("keygens", mapping=__extras)
+keygens = Namespace.make_module("keygens", mapping=__extras)
 
